@@ -98,17 +98,17 @@ typedef struct svn_auth_provider_t
    * provider may need, and @a realmstring comes from the
    * svn_auth_first_credentials() call.
    */
-  svn_error_t * (*first_credentials) (void **credentials,
-                                      void **iter_baton,
-                                      void *provider_baton,
-                                      apr_hash_t *parameters,
-                                      const char *realmstring,
-                                      apr_pool_t *pool);
+  svn_error_t * (*first_credentials)(void **credentials,
+                                     void **iter_baton,
+                                     void *provider_baton,
+                                     apr_hash_t *parameters,
+                                     const char *realmstring,
+                                     apr_pool_t *pool);
 
   /** Get a different set of credentials.
    *
-   * Set @a *credentials to another set of valid credentials, (using
-   * @a iter_baton as the context from previous call to first_credentials
+   * Set @a *credentials to another set of valid credentials (using @a
+   * iter_baton as the context from previous call to first_credentials
    * or next_credentials).  If no more credentials are available, set
    * @a *credentials to NULL.  If the provider only has one set of
    * credentials, this function pointer should simply be NULL. @a
@@ -116,12 +116,12 @@ typedef struct svn_auth_provider_t
    * contains any run-time data that the provider may need, and @a
    * realmstring comes from the svn_auth_first_credentials() call.
    */
-  svn_error_t * (*next_credentials) (void **credentials,
-                                     void *iter_baton,
-                                     void *provider_baton,
-                                     apr_hash_t *parameters,
-                                     const char *realmstring,
-                                     apr_pool_t *pool);
+  svn_error_t * (*next_credentials)(void **credentials,
+                                    void *iter_baton,
+                                    void *provider_baton,
+                                    apr_hash_t *parameters,
+                                    const char *realmstring,
+                                    apr_pool_t *pool);
   
   /** Save credentials.
    *
@@ -134,12 +134,12 @@ typedef struct svn_auth_provider_t
    * this function pointer should simply be NULL. @a realmstring comes
    * from the svn_auth_first_credentials() call.
    */
-  svn_error_t * (*save_credentials) (svn_boolean_t *saved,
-                                     void *credentials,
-                                     void *provider_baton,
-                                     apr_hash_t *parameters,
-                                     const char *realmstring,
-                                     apr_pool_t *pool);
+  svn_error_t * (*save_credentials)(svn_boolean_t *saved,
+                                    void *credentials,
+                                    void *provider_baton,
+                                    apr_hash_t *parameters,
+                                    const char *realmstring,
+                                    apr_pool_t *pool);
   
 } svn_auth_provider_t;
 
@@ -299,8 +299,9 @@ typedef struct svn_auth_ssl_server_cert_info_t
  *
  * @since New in 1.3.
  */
-svn_auth_ssl_server_cert_info_t *svn_auth_ssl_server_cert_info_dup (
-  const svn_auth_ssl_server_cert_info_t *info, apr_pool_t *pool);
+svn_auth_ssl_server_cert_info_t *
+svn_auth_ssl_server_cert_info_dup(const svn_auth_ssl_server_cert_info_t *info,
+                                  apr_pool_t *pool);
 
 /** @c SVN_AUTH_CRED_SSL_SERVER_TRUST credentials. */
 typedef struct svn_auth_cred_ssl_server_trust_t
@@ -336,7 +337,7 @@ typedef struct svn_auth_cred_ssl_server_trust_t
  * If @a realm is non-null, maybe use it in the prompt string.
  *
  * If @a username is non-null, then the user might be prompted only
- * for a password, but @a *creds would still be filled with both
+ * for a password, but @a *cred would still be filled with both
  * username and password.  For example, a typical usage would be to
  * pass @a username on the first call, but then leave it null for
  * subsequent calls, on the theory that if credentials failed, it's
@@ -348,13 +349,13 @@ typedef struct svn_auth_cred_ssl_server_trust_t
  * client with a remember password checkbox would grey out the checkbox if
  * @a may_save is FALSE.
  */
-typedef svn_error_t *
-(*svn_auth_simple_prompt_func_t) (svn_auth_cred_simple_t **cred,
-                                  void *baton,
-                                  const char *realm,
-                                  const char *username,
-                                  svn_boolean_t may_save,
-                                  apr_pool_t *pool);
+typedef svn_error_t *(*svn_auth_simple_prompt_func_t)
+  (svn_auth_cred_simple_t **cred,
+   void *baton,
+   const char *realm,
+   const char *username,
+   svn_boolean_t may_save,
+   apr_pool_t *pool);
 
 
 /** Set @a *cred by prompting the user, allocating @a *cred in @a pool.
@@ -368,12 +369,12 @@ typedef svn_error_t *
  * client with a remember username checkbox would grey out the checkbox if
  * @a may_save is FALSE.
  */
-typedef svn_error_t *
-(*svn_auth_username_prompt_func_t) (svn_auth_cred_username_t **cred,
-                                    void *baton,
-                                    const char *realm,
-                                    svn_boolean_t may_save,
-                                    apr_pool_t *pool);
+typedef svn_error_t *(*svn_auth_username_prompt_func_t)
+  (svn_auth_cred_username_t **cred,
+   void *baton,
+   const char *realm,
+   svn_boolean_t may_save,
+   apr_pool_t *pool);
 
 
 /** @name SSL server certificate failure bits
@@ -411,14 +412,14 @@ typedef svn_error_t *
  * client with a trust permanently checkbox would grey out the checkbox if
  * @a may_save is FALSE.
  */
-typedef svn_error_t *(*svn_auth_ssl_server_trust_prompt_func_t) (
-  svn_auth_cred_ssl_server_trust_t **cred,
-  void *baton,
-  const char *realm,
-  apr_uint32_t failures,
-  const svn_auth_ssl_server_cert_info_t *cert_info,
-  svn_boolean_t may_save,
-  apr_pool_t *pool);
+typedef svn_error_t *(*svn_auth_ssl_server_trust_prompt_func_t)
+  (svn_auth_cred_ssl_server_trust_t **cred,
+   void *baton,
+   const char *realm,
+   apr_uint32_t failures,
+   const svn_auth_ssl_server_cert_info_t *cert_info,
+   svn_boolean_t may_save,
+   apr_pool_t *pool);
 
 
 /** Set @a *cred by prompting the user, allocating @a *cred in @a pool.
@@ -431,12 +432,12 @@ typedef svn_error_t *(*svn_auth_ssl_server_trust_prompt_func_t) (
  * client with a remember certificate checkbox would grey out the checkbox
  * if @a may_save is FALSE.
  */
-typedef svn_error_t *(*svn_auth_ssl_client_cert_prompt_func_t) (
-  svn_auth_cred_ssl_client_cert_t **cred,
-  void *baton,
-  const char *realm,
-  svn_boolean_t may_save,
-  apr_pool_t *pool);
+typedef svn_error_t *(*svn_auth_ssl_client_cert_prompt_func_t)
+  (svn_auth_cred_ssl_client_cert_t **cred,
+   void *baton,
+   const char *realm,
+   svn_boolean_t may_save,
+   apr_pool_t *pool);
 
 
 /** Set @a *cred by prompting the user, allocating @a *cred in @a pool.
@@ -449,12 +450,12 @@ typedef svn_error_t *(*svn_auth_ssl_client_cert_prompt_func_t) (
  * client with a remember password checkbox would grey out the checkbox if
  * @a may_save is FALSE.
  */
-typedef svn_error_t *(*svn_auth_ssl_client_cert_pw_prompt_func_t) (
-  svn_auth_cred_ssl_client_cert_pw_t **cred,
-  void *baton,
-  const char *realm,
-  svn_boolean_t may_save,
-  apr_pool_t *pool);
+typedef svn_error_t *(*svn_auth_ssl_client_cert_pw_prompt_func_t)
+  (svn_auth_cred_ssl_client_cert_pw_t **cred,
+   void *baton,
+   const char *realm,
+   svn_boolean_t may_save,
+   apr_pool_t *pool);
 
 
 
@@ -476,8 +477,8 @@ void svn_auth_open(svn_auth_baton_t **auth_baton,
  *
  * Store @a name / @a value pair as a run-time parameter in @a
  * auth_baton, making the data accessible to all providers.  @a name
- * and @a value will be NOT be duplicated into the auth_baton's
- * pool. To delete a run-time parameter, pass NULL for @a value.
+ * and @a value will NOT be duplicated into the auth_baton's pool.
+ * To delete a run-time parameter, pass NULL for @a value.
  */
 void svn_auth_set_parameter(svn_auth_baton_t *auth_baton,
                             const char *name,
@@ -595,6 +596,219 @@ svn_error_t * svn_auth_save_credentials(svn_auth_iterstate_t *state,
 
 /** @} */
 
+/** Create and return @a *provider, an authentication provider of type
+ * svn_auth_cred_simple_t that gets information by prompting the user
+ * with @a prompt_func and @a prompt_baton.  Allocate @a *provider in
+ * @a pool.
+ *
+ * If both @c SVN_AUTH_PARAM_DEFAULT_USERNAME and
+ * @c SVN_AUTH_PARAM_DEFAULT_PASSWORD are defined as runtime
+ * parameters in the @c auth_baton, then @a *provider will return the
+ * default arguments when svn_auth_first_credentials() is called.  If
+ * svn_auth_first_credentials() fails, then @a *provider will
+ * re-prompt @a retry_limit times (via svn_auth_next_credentials()).
+ *
+ * @since New in 1.4.
+ */
+void
+svn_auth_get_simple_prompt_provider(svn_auth_provider_object_t **provider,
+                                    svn_auth_simple_prompt_func_t prompt_func,
+                                    void *prompt_baton,
+                                    int retry_limit,
+                                    apr_pool_t *pool);
+
+
+/** Create and return @a *provider, an authentication provider of type @c
+ * svn_auth_cred_username_t that gets information by prompting the
+ * user with @a prompt_func and @a prompt_baton.  Allocate @a *provider
+ * in @a pool.
+ *
+ * If @c SVN_AUTH_PARAM_DEFAULT_USERNAME is defined as a runtime
+ * parameter in the @c auth_baton, then @a *provider will return the
+ * default argument when svn_auth_first_credentials() is called.  If
+ * svn_auth_first_credentials() fails, then @a *provider will
+ * re-prompt @a retry_limit times (via svn_auth_next_credentials()).
+ *
+ * @since New in 1.4.
+ */
+void svn_auth_get_username_prompt_provider
+  (svn_auth_provider_object_t **provider,
+   svn_auth_username_prompt_func_t prompt_func,
+   void *prompt_baton,
+   int retry_limit,
+   apr_pool_t *pool);
+
+
+/** Create and return @a *provider, an authentication provider of type @c
+ * svn_auth_cred_simple_t that gets/sets information from the user's
+ * ~/.subversion configuration directory.  Allocate @a *provider in
+ * @a pool.
+ *  
+ * If a default username or password is available, @a *provider will
+ * honor them as well, and return them when
+ * svn_auth_first_credentials() is called.  (see @c
+ * SVN_AUTH_PARAM_DEFAULT_USERNAME and @c
+ * SVN_AUTH_PARAM_DEFAULT_PASSWORD). 
+ *
+ * @since New in 1.4.
+ */
+void svn_auth_get_simple_provider(svn_auth_provider_object_t **provider,
+                                  apr_pool_t *pool);
+
+
+#if defined(WIN32) || defined(DOXYGEN)
+/**
+ * Create and return @a *provider, an authentication provider of type @c
+ * svn_auth_cred_simple_t that gets/sets information from the user's
+ * ~/.subversion configuration directory.  Allocate @a *provider in
+ * @a pool.
+ *
+ * This is like svn_client_get_simple_provider(), except that, when
+ * running on Window 2000 or newer (or any other Windows version that
+ * includes the CryptoAPI), the provider encrypts the password before
+ * storing it to disk. On earlier versions of Windows, the provider
+ * does nothing.
+ *
+ * @since New in 1.4.
+ * @note This function is only available on Windows.
+ *
+ * @note An administrative password reset may invalidate the account's
+ * secret key. This function will detect that situation and behave as
+ * if the password were not cached at all.
+ */
+void
+svn_auth_get_windows_simple_provider(svn_auth_provider_object_t **provider,
+                                     apr_pool_t *pool);
+#endif /* WIN32 || DOXYGEN */
+
+#if defined(DARWIN) || defined(DOXYGEN)
+
+/**
+ * Create and return @a *provider, an authentication provider of type @c
+ * svn_auth_cred_simple_t that gets/sets information from the user's
+ * ~/.subversion configuration directory.  Allocate @a *provider in
+ * @a pool.
+ *
+ * This is like svn_client_get_simple_provider(), except that the
+ * password is stored in the Mac OS KeyChain.
+ *
+ * @since New in 1.4
+ * @note This function is only available on Mac OS 10.2 and higher.
+ */
+void
+svn_auth_get_keychain_simple_provider(svn_auth_provider_object_t **provider,
+                                      apr_pool_t *pool);
+
+#endif /* DARWIN || DOXYGEN */
+
+/** Create and return @a *provider, an authentication provider of type @c
+ * svn_auth_cred_username_t that gets/sets information from a user's
+ * ~/.subversion configuration directory.  Allocate @a *provider in
+ * @a pool.
+ *
+ * If a default username is available, @a *provider will honor it,
+ * and return it when svn_auth_first_credentials() is called.  (See
+ * @c SVN_AUTH_PARAM_DEFAULT_USERNAME.)
+ *
+ * @since New in 1.4.
+ */
+void svn_auth_get_username_provider(svn_auth_provider_object_t **provider,
+                                    apr_pool_t *pool);
+
+
+/** Create and return @a *provider, an authentication provider of type @c
+ * svn_auth_cred_ssl_server_trust_t, allocated in @a pool.
+ *
+ * @a *provider retrieves its credentials from the configuration
+ * mechanism.  The returned credential is used to override SSL
+ * security on an error.
+ *
+ * @since New in 1.4.
+ */
+void svn_auth_get_ssl_server_trust_file_provider
+  (svn_auth_provider_object_t **provider,
+   apr_pool_t *pool);
+
+
+/** Create and return @a *provider, an authentication provider of type @c
+ * svn_auth_cred_ssl_client_cert_t, allocated in @a pool.
+ *
+ * @a *provider retrieves its credentials from the configuration
+ * mechanism.  The returned credential is used to load the appropriate
+ * client certificate for authentication when requested by a server.
+ *
+ * @since New in 1.4.
+ */
+void svn_auth_get_ssl_client_cert_file_provider
+  (svn_auth_provider_object_t **provider,
+   apr_pool_t *pool);
+
+
+/** Create and return @a *provider, an authentication provider of type @c
+ * svn_auth_cred_ssl_client_cert_pw_t, allocated in @a pool.
+ *
+ * @a *provider retrieves its credentials from the configuration
+ * mechanism.  The returned credential is used when a loaded client
+ * certificate is protected by a passphrase.
+ *
+ * @since New in 1.4.
+ */
+void svn_auth_get_ssl_client_cert_pw_file_provider
+  (svn_auth_provider_object_t **provider,
+   apr_pool_t *pool);
+
+
+/** Create and return @a *provider, an authentication provider of type @c
+ * svn_auth_cred_ssl_server_trust_t, allocated in @a pool.  
+ *
+ * @a *provider retrieves its credentials by using the @a prompt_func
+ * and @a prompt_baton.  The returned credential is used to override
+ * SSL security on an error.
+ *
+ * @since New in 1.4.
+ */
+void svn_auth_get_ssl_server_trust_prompt_provider
+  (svn_auth_provider_object_t **provider,
+   svn_auth_ssl_server_trust_prompt_func_t prompt_func,
+   void *prompt_baton,
+   apr_pool_t *pool);
+
+
+/** Create and return @a *provider, an authentication provider of type @c
+ * svn_auth_cred_ssl_client_cert_t, allocated in @a pool.
+ *
+ * @a *provider retrieves its credentials by using the @a prompt_func
+ * and @a prompt_baton.  The returned credential is used to load the
+ * appropriate client certificate for authentication when requested by
+ * a server.  The prompt will be retried @a retry_limit times.
+ *
+ * @since New in 1.4.
+ */
+void svn_auth_get_ssl_client_cert_prompt_provider
+  (svn_auth_provider_object_t **provider,
+   svn_auth_ssl_client_cert_prompt_func_t prompt_func,
+   void *prompt_baton,
+   int retry_limit,
+   apr_pool_t *pool);
+
+
+/** Create and return @a *provider, an authentication provider of type @c
+ * svn_auth_cred_ssl_client_cert_pw_t, allocated in @a pool.
+ *
+ * @a *provider retrieves its credentials by using the @a prompt_func
+ * and @a prompt_baton.  The returned credential is used when a loaded
+ * client certificate is protected by a passphrase.  The prompt will
+ * be retried @a retry_limit times.
+ *
+ * @since New in 1.4.
+ */
+void svn_auth_get_ssl_client_cert_pw_prompt_provider
+  (svn_auth_provider_object_t **provider,
+   svn_auth_ssl_client_cert_pw_prompt_func_t prompt_func,
+   void *prompt_baton,
+   int retry_limit,
+   apr_pool_t *pool);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
